import React, { useState } from "react";
import { Tree, Icon, Card } from "antd";
import { nanoid } from "nanoid";//生成唯一id
import { duration } from "moment";

const { TreeNode } = Tree;
//树形数据
const treeData = [
  {
    value: "0",
    defaultValue: "0",
    key: "0",
    parentKey: '0',
    isEditable: false,
    children: [
      {
        value: "0",
        key: "0-1",
        defaultValue: "0-1",
        isEditable: false,
      },
    ],
  },

];

const expandedKeyArr = ["0"];
export default function TreeDemo() {
  const [data, setData] = useState(treeData);//获取值，及时更新被修改的值。变量
  const [expandedKeys, setExpandedKeys] = useState(expandedKeyArr);//状态
  const onExpand = (expandedKeys) => {
    //记录折叠的key值
    setExpandedKeys(expandedKeys);
  };
  //开始渲染树形节点 data拿到数据
  const renderTreeNodes = (data) => {
    //遍历项目
    let nodeArr = data.map((item) => {
      //项目编辑内容=title
      if (item.isEditable) {
        item.title = (
          <div>
            {/* 逻辑运算 OR  */}
            {/* item项目value根据状态切换 */}
            {/*onChange这个API是一个函数，并且需要返回值，API的返回值是onChange方法的返回值
            e就是api的形参，这个参数的生成需要onChange的API触发，同时将e传给方法并获取其返回值
            实参把值给形参，由形参保存。

            这个e会获取到最新的变化值并且传到onchange方法
            */}
            <input value={item.value || ''} onChange={(e) => onChange(e, item.key)} />

            <Icon
              //取消保存
              type="close-circle"
              theme="filled"

              style={{ marginLeft: 10 }}
              onClick={() => onClose(item.key, item.defaultValue)}//恢复默认值 
            />
            <Icon
              //保存节点
              type="check-circle"
              theme="filled"
              style={{ marginLeft: 10 }}
              onClick={() => onSave(item.key)}
            />
          </div>
        );
      } else {
        //放在标题旁边
        item.title = (
          <div >
            <span>{item.value}</span>
            <span>
              {/* 编辑edit onclick 拿到项目的key */}
              <Icon
                type="edit"
                style={{ marginLeft: 10 }}
                onClick={() => onEdit(item.key)}
              />
              {/* 编辑plus onclick 拿到项目的key */}
              <Icon
                type="plus"
                style={{ marginLeft: 10 }}
                onClick={() => onAdd(item.key)}
              />
              {/* 先确认父key是不是绝对等于"0" */}
              {item.parentKey === "0" ? null : (
                <Icon
                  type="delete"
                  style={{ marginLeft: 10 }}
                  onClick={() => onDelete(item.key)}
                />
              )}
            </span>
          </div>
        );
      }
      //更新数据----------------------------
      if (item.children) {
        return (
          <TreeNode title={item.title} key={item.key} dataRef={item}>
            {/* 渲染 */}
            {renderTreeNodes(item.children)}
          </TreeNode>
        );
      }
      //返回他的标题，和key值
      return <TreeNode title={item.title} key={item.key} />;
    });
    //返回节点
    return nodeArr;
  }
  //新增
  const onAdd = (key) => {
    if (expandedKeys.indexOf(key) === -1) {
      expandedKeyArr.push(key);
    }
    setExpandedKeys(expandedKeyArr.slice());
    addNode(key, treeData);//添加节点 树形
    //useState里数据务必为immutable （不可赋值的对象），所以必须加上slice()返回一个新的数组对象
    setData(treeData.slice());//添加子项
  };
  //编辑
  //拿到key treeData数据--setData设置数据---slice删除数据中已返回的元素  [aa,bb,cc,dd,ee]--slice(2)输出[cc,dd,ee]
  const onEdit = (key) => {
    editNode(key, treeData);
    setData(treeData.slice());
  };
  const editNode = (key, data) =>
    data.forEach((item) => {
      if (item.key === key) {
        //编辑状态 true

        item.isEditable = true;
      } else {

        //编辑状态 false
        item.isEditable = false;
      }
      item.value = item.defaultValue; // 当某节点处于编辑状态，并改变数据，点击编辑其他节点时，此节点变成不可编辑状态，value 需要回退到 defaultvalue
      //当某子项处于编辑状态，改变数据，点击其他时候，此项变成不可编辑
      if (item.children) {
        editNode(key, item.children);
        //编辑子节点
      }
    });

  const addNode = (key, data) =>
    data.forEach((item) => {
      if (item.key === key) {//如果拿到的key相同则判断子项
        if (item.children) {
          item.children.push({
            value: "default",//创建新的子项默认值
            key: nanoid(), // 这个 key 应该是唯一的
          });
        } else {
          item.children = [];
          item.children.push({
            value: "default",
            key: nanoid(),
          });
        }
        return;
      }
      if (item.children) {
        addNode(key, item.children);
      }
    });

  const onChange = (e, key) => {
    changeNode(key, e.target.value, treeData);
    setData(treeData.slice());
  };

  const changeNode = (key, value, data) =>
    data.forEach((item) => {
      if (item.key === key) {//新增子项内容赋值
        item.value = value;
      }
      if (item.children) {
        changeNode(key, value, item.children);//子项
      }
    });

  const onSave = (key) => {
    saveNode(key, treeData);
    setData(treeData.slice());
  };
  //确认保存节点
  const saveNode = (key, data) =>
    data.forEach((item) => {
      if (item.key === key) {
        item.defaultValue = item.value;//item.value内容赋值给defaultValue
      }
      //子项保存不能编辑
      if (item.children) {
        saveNode(key, item.children);
      }
      //项目是否能编辑
      item.isEditable = false;
    });

  const onClose = (key, defaultValue) => {
    closeNode(key, defaultValue, treeData);
    //项目关闭，获取treeData树数据值
    setData(treeData);
  };
  //取消确认
  const closeNode = (key, defaultValue, data) =>
    data.forEach((item) => {
      item.isEditable = false;
      if (item.key === key) {
        item.value = defaultValue;
      }
      if (item.children) {
        closeNode(key, defaultValue, item.children);
      }
    });
  // slice() 方法可提取字符串的某个部分，并以新的字符串返回被提取的部分。
  const onDelete = (key) => {
    deleteNode(key, treeData);
    setData(treeData.slice());
  };
  //删除节点
  const deleteNode = (key, data) =>
    //拿到每个key数据， 如果要删除的key相同，则删除，并返回。如果不同判断子项
    data.forEach((item, index) => {
      if (item.key === key) {
        data.splice(index, 1);//从某个位置开始删除元素
        return;
      } else {
        if (item.children) {
          deleteNode(key, item.children);
        }
      }
    });

  return (
    <div>
      <Card style={{ width: '90%' }}>
        <Tree
          switcherIcon={<Icon type="down-square" theme="filled" />}
          expandedKeys={expandedKeys}
          onExpand={onExpand}>
          {renderTreeNodes(data)}
        </Tree>
      </Card>

    </div>
  );
}